#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

// global rubberspoon object.
#include "gtkgui.h"

#include "rs043.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

const gchar *in_filename;

void
on_load_image_open_clicked             (gpointer         user_data,
                                        GtkButton       *button)
{
	in_filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (user_data));
	//g_print ("%s\n", filename);
	gtk_widget_destroy(GTK_WIDGET(user_data));
	GtkImage *image_input = GTK_IMAGE(lookup_widget(rubberspoon, "image_input"));
	gtk_image_set_from_file(image_input, in_filename);
}

void
on_load_image_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *filew;
	filew = create_load_image_dialog ();
	gtk_widget_show (filew);
}

void load_rc(gchar *site, gchar *user, gchar *type)
{
	// TODO: Make this support win32.
	FILE *rc = fopen("/home/jvoss/.rs/rsrc.txt", "r");
	if(rc == 0)
	{
		// No RC file, so give them the defaults.
		strcpy(site, "https://www.altsci.com/rs/");
		strcpy(user, "");
		strcpy(type, "Rubber Spoon");
		return;
	}
	else
	{
		// RC file, so give them their saved settings.
		// A proper rsrc.txt file looks like:
/*
https://www.altsci.com/rs/
joe_user
Rubber Spoon
*/
		fgets(site, 1024, rc);
		site[strlen(site) - 1] = 0;
		fgets(user, 256,  rc);
		user[strlen(user) - 1] = 0;
		fgets(type, 256,  rc);
		type[strlen(type) - 1] = 0;
		fclose(rc);
	}
}

void save_rc(const gchar *site, const gchar *user, const gchar *type)
{
	g_print ("%s, %s, %s.\n", site, user, type);
	// TODO: Make this support win32 and other users.
	struct stat buf;
	if(stat("/home/jvoss/.rs", &buf) != 0)
	{
		// Create the directory.
		if(mkdir("/home/jvoss/.rs", 0700) != 0)
		{
			g_print ("Error, resource file directory cannnot be created.\n");
			return;
		}
	}
	else
	{
		if(!S_ISDIR(buf.st_mode))
		{
			g_print ("Error, resource file directory is used.\n");
			return;
		}
	}
	FILE *rc = fopen("/home/jvoss/.rs/rsrc.txt", "w");
	if(rc == 0)
	{
		g_print ("Error, cannot save resource file.\n");
		return;
	}
	fputs(site, rc);
	fputs("\n", rc);
	fputs(user, rc);
	fputs("\n", rc);
	fputs(type, rc);
	fputs("\n", rc);
	fclose(rc);
}

void
on_settings_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *settings_dialog;
	settings_dialog = create_settings_dialog ();
	gtk_widget_show (settings_dialog);
	
	GtkEntry *settings_site = GTK_ENTRY(lookup_widget(settings_dialog, "settings_site"));
	GtkEntry *settings_user = GTK_ENTRY(lookup_widget(settings_dialog, "settings_user"));
	GtkEntry *settings_type = GTK_ENTRY(lookup_widget(settings_dialog, "settings_type"));
	gchar site[1024], user[256], type[256];
	load_rc(site, user, type);
	gtk_entry_set_text(settings_site, site);
	gtk_entry_set_text(settings_user, user);
	gtk_entry_set_text(settings_type, type);
}


void
on_upload_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *password_dialog;
	password_dialog = create_password_dialog ();
	gtk_widget_show (password_dialog);
}


void
on_settings_ok_clicked                 (gpointer         user_data,
                                        GtkButton       *button)
{
	// How do I get the data from 3+ text boxes?
	GtkWidget *settings_dialog = GTK_WIDGET(user_data);
	GtkEntry *settings_site = GTK_ENTRY(lookup_widget(settings_dialog, "settings_site"));
	GtkEntry *settings_user = GTK_ENTRY(lookup_widget(settings_dialog, "settings_user"));
	GtkEntry *settings_type = GTK_ENTRY(lookup_widget(settings_dialog, "settings_type"));
	const gchar *site = gtk_entry_get_text(settings_site);
	const gchar *user = gtk_entry_get_text(settings_user);
	const gchar *type = gtk_entry_get_text(settings_type);
	save_rc(site, user, type);

	gtk_widget_destroy(settings_dialog);
}


void
on_settings_cancel_clicked             (gpointer         user_data,
                                        GtkButton       *button)
{

	gtk_widget_destroy(GTK_WIDGET(user_data));
}


void free_out_data(guchar *pixels, gpointer data)
{
	g_free(pixels);
}

void
on_password_ok_clicked                 (gpointer         user_data,
                                        GtkButton       *button)
{
	GtkWidget *password_dialog = GTK_WIDGET(user_data);
	
	// Steg the image and load the preview.
	//g_print ("steg upload.\n");
	
	// The plan: 
	// 1) Take the data from g_input_image.
	GtkImage *image_input = GTK_IMAGE(lookup_widget(rubberspoon, "image_input"));
	GdkPixbuf *pixbuf;
	pixbuf = gtk_image_get_pixbuf (image_input);
	
	// 2) Get the text to steg
	GtkTextView *message = GTK_TEXT_VIEW(lookup_widget(rubberspoon, "message"));
	gchar *data;
	int data_length;
	GtkTextBuffer *message_buf = gtk_text_view_get_buffer (message);
	GtkTextIter start_iter, end_iter;
	gtk_text_buffer_get_start_iter (message_buf, &start_iter);
	gtk_text_buffer_get_end_iter (message_buf, &end_iter);
	data = gtk_text_buffer_get_text (message_buf, &start_iter, &end_iter, TRUE);
	data_length = strlen(data);
	
	// 3) Get a password.
	int key[50];
	
	GtkEntry *password_input = GTK_ENTRY(lookup_widget(password_dialog, "password_input"));
	const gchar *password = gtk_entry_get_text(password_input);
	
	// Secret Key
	StegKey(password, key);
	gtk_widget_destroy(password_dialog);
	
	// 4) Execute the steg function.
	guchar *pixels;
	int width, height, rowstride, n_channels, image_size;
	n_channels = gdk_pixbuf_get_n_channels (pixbuf);
	width = gdk_pixbuf_get_width (pixbuf);
	height = gdk_pixbuf_get_height (pixbuf);
	pixels = gdk_pixbuf_get_pixels (pixbuf);
	image_size = n_channels * width * height;
	
	guchar *out_data = g_new(guchar, image_size);

	//printf("%i, %s, %i, %08x, %i, %08x, %08x;\n", n_channels, data, data_length, key, 50, pixels, out_data);
	steg(data, data_length, key, 50, pixels, out_data);
	// Copy the data after real_data_length.
	// bytes_stegged = size * 8 bits * key[key_size - 1] per 50 bits
	int real_size = data_length * 8 * (key[49]+1) / 50;
	memcpy(out_data + real_size, pixels + real_size, image_size-real_size);
	
	// Output the file as ppm 24-bit.
	char out_filename[1024];
	strncpy(out_filename, in_filename, 1023);
	// Ensure null termination.
	out_filename[1023] = 0;
	char *ext = strstr(out_filename, ".");
	if((ext < out_filename) || (ext - out_filename > 1016))
	{
		ext = out_filename + strlen(out_filename);
	}
	if((ext >= out_filename) && (ext - out_filename < 1016))
	{
		strncpy(ext, "-1.ppm", 7);
	}
	else
	{
		strncpy(out_filename, "out.ppm", 1023);
	}
	FILE *out_file = fopen(out_filename, "wb");
	fprintf(out_file, "P6\n%i %i\n255\n", width, height);
	fwrite(out_data, image_size, 1, out_file);
	fclose(out_file);
	g_print("Wrote stegged image: %s.\n", out_filename);
	
	// 5) Draw it in the output.
	GtkImage *output_image = GTK_IMAGE(lookup_widget(rubberspoon, "output_image"));
	GdkPixbuf *out_pixbuf = gdk_pixbuf_new_from_data(out_data, 
		GDK_COLORSPACE_RGB, FALSE, 8, width, height, width * n_channels, 
		free_out_data, NULL);
	gtk_image_set_from_pixbuf(output_image, out_pixbuf);
	
	GtkWidget *success_dialog;
	success_dialog = create_success_dialog ();
	gtk_widget_show (success_dialog);
}

void
on_password_cancel_clicked             (gpointer         user_data,
                                        GtkButton       *button)
{

	gtk_widget_destroy(GTK_WIDGET(user_data));
}


void
on_rubberspoon_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_main_quit();
}



void
on_image_input_realize                 (GtkWidget       *widget,
                                        gpointer         user_data)
{
	g_print ("g_image_input is set.\n");

}


gboolean
on_output_image_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	// Steg the image and load the preview.
	g_print ("TODO: steg preview.\n");
	
	// The plan: 
	// 1) Take the data from g_input_image.
	GtkImage *image_input = GTK_IMAGE(lookup_widget(rubberspoon, "image_input"));
	GdkPixmap *pixmap = NULL;
	GdkBitmap *mask = NULL;
	gtk_image_get_pixmap (image_input, &pixmap, &mask);
	
	// 2) Get the text to steg
	GtkTextView *message = GTK_TEXT_VIEW(lookup_widget(rubberspoon, "message"));
	
	// 3) Get a password.
	GtkWidget *password_dialog;
	password_dialog = create_password_dialog ();
	gtk_widget_show (password_dialog);
	
	// NOTE: The rest is done above in the passwork ok click.
	// 4) Execute the steg function.
	
	// 5) Draw it in the output.
	
	/*
	gdk_pixmap_create_from_data(GdkDrawable *drawable, 
		const gchar *data, gint width, gint height, gint depth, 
		const GdkColor *fg, const GdkColor *bg)
	gtk_image_set_from_pixmap (GtkImage *image, GdkPixmap *pixmap, GdkBitmap *mask)
	*/
	return FALSE;
}

