/*
 * Rubber Spoon libSteg 0.4
 * by Joel R. Voss
 * August 16, 2004 10:55
 * SEE rs04.txt for more details.
 */

#include "rs04.h"

// 1a) data bits -> data byte
// convertBitToByte converts one bit to a byte with the bit in the lowest 
// position. if bit is true, 1, else 0.
BYTE convertBitToByte(BYTE input, BYTE bit)
{
	// Bit must be between 0-7, else throw error.
	if(bit > 7) return 0;
	BYTE mask = 0x1 << bit;
	//printf("mask = %x;\n", mask);
	return (input & mask) >> bit;
}

// 1b) data bits -> data byte
// convertHexToBinary converts a byte to 8 bytes with each bit in the lowest 
// position of the byte it created.
void convertHexToBinary(BYTE data, BYTE *dest)
{
	int bit;
	//printf("data = %x;\n", data);
	for(bit = 7; bit >= 0; bit--)
	{
		dest[7-bit] = convertBitToByte(data, bit);
		//printf("output = %x;\n", output);
	}
}


// 1c) data bits -> data byte
// convertDataToBinary converts a N bytes to 8 * N bytes with each bit in the 
// lowest position of the byte it created.
void convertDataToBinary(BYTE *data, DWORD size, BYTE *dest)
{
	int index;
	//printf("data = %x;\n", data);
	for(index = 0; index < size; index++)
	{
		convertHexToBinary(data[index], &(dest[8*index]));
		//printf("output = %x;\n", output);
	}
}

// 2) Mask
// createMask will become an ultimate mask generator system.
// for now, it simply writes size bytes of 0x1 = binary 00000001.
void createMask(BYTE *dest, BYTE data, DWORD size)
{
	int index;
	//printf("data = %x;\n", data);
	for(index = 0; index < size; index++)
	{
		dest[index] = data;
	}
}

// 3a) dest1 = pixels and inverse mask
// andData ands all the data in a and b.
void andData(BYTE *a, BYTE *b, DWORD size, BYTE *dest)
{
	int index;
	//printf("and = %x = %x & %x;\n", dest, a, b);
	for(index = 0; index < size; index++)
	{
		dest[index] = a[index] & b[index];
		//printf("output[%i] = %x;\n", index, dest[index]);
	}
}

// 3b) dest2 = dest1 or data bytes
// orData ors all the data in a and b.
void orData(BYTE *a, BYTE *b, DWORD size, BYTE *dest)
{
	int index;
	//printf("or  = %x = %x | %x;\n", dest, a, b);
	for(index = 0; index < size; index++)
	{
		dest[index] = a[index] | b[index];
		//printf("output[%i] = %x;\n", index, dest[index]);
	}
}

// steg: A simple function to take data and interlace it with pixels of an 
// image such that an onlooker will not see the data. <1>
void steg(BYTE *data, DWORD size, BYTE *pixels, BYTE *output)
{
	BYTE    bytes[size * 8];
	BYTE inv_mask[size * 8];
	
	// 1) data bits -> data bytes
	convertDataToBinary(data, size, bytes);
	
	// Inverse Mask = 0xfe = 254 = 11111110
	createMask(inv_mask, 0xfe, size * 8);
	
	// And Data with inv_mask deletes the least significant bit.
	andData(pixels, inv_mask, size * 8, output);
	
	// Or Data with lsb data puts the data in the least significant bit.
	orData(output, bytes, size * 8, output);

}

#ifdef TEST

#include <stdio.h>

// Test libSteg steg 0.4
int main(int argc, char *argv)
{
	BYTE data[] = {231, 14, 32, 16, 77};
	int size = 5;
	BYTE   pixels[size * 8];
	BYTE   output[size * 8];
	
	// Get a suitable image.
	fread(pixels, size * 8, 1, stdin);
	
	// The actual stegging.
	steg(data, size, pixels, output);
	
	// Produce a copy of the image that contains the stegged data.
	fputs("P5\n8 5\n255\n", stdout);
	fwrite(output, size * 8, 1, stdout);
	//printf("\n");
	return 0;
}
#endif // TEST

/* <1> However, this simple 
 * function could be broken by someone simply looking at the least significant 
 * bit and concatenating them. A good steg system uses a key large enough that 
 * a person can deny knowledge of the secret data. If the data is vague enough, 
 * and likely enough to occur using random keys, it is possible to deny it. <2>
 *
 * <2> For example, if I had a stegged picture with a good key, a person could 
 * brute force the key and get several messages: one might say "meet me at 6" 
 * another might say, "terror at 7 pm", while another might say "i R 1337". 
 * Which is the most likely? Probably the third.
 */
